<?php

namespace App\Factory;

use App\Exception;

class OrderFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util)
    {

        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function insertSale ($sale, $generalParams, $order, $consumer)
    {
        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    			
    	date_default_timezone_set($this->getCurrentTimezone($generalParams["SGESTADO"]));
    	
    	$params = [
    	    ":CDFILIAL"     => $generalParams["CDFILIAL"],
            ":NRVENDAREST"  => $sale,
            ":CDLOJA"       => $generalParams["CDLOJA"],
            ":NRMESA"       => "",
            ":CDVENDEDOR"   => $generalParams["CDVENDEDOR"],
            ":CURRDATE"     => date("d/m/Y H:i:s"),
            ":CDOPERADOR"   => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN" => "1",
            ":CDCLIENTE"    => $consumer["CDCLIENTE"],
            ":CDCONSUMIDOR" => $consumer["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand ($generalParams, $order, $consumer, $deliveryFee)
    {
    	$sql = "
    		INSERT INTO COMANDAVEN 
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    			CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    			TXMOTIVCANCE, VRACRCOMANDA, CDPAIS, SGESTADO, 
    			CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO,
    			NRENDECONSCOMAN, DSENDECONSCOMAN, DSCOMPLENDCOCOM,
    			DSREFENDCONSCOM, DSOBSCOMANDA, IDORGCMDVENDA, IDRETBALLOJA,
    			NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO, DSAREAATEND,
    			IDCOMANDAEXT, NRCONTATOCON, DSCONSUMIDOR, IDIMPCUPEXP)
    		VALUES 
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    			:CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    			:TXMOTIVCANCE, :VRACRCOMANDA, :CDPAIS, :SGESTADO, 
    			:CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO,
    			:NRENDECONSCOMAN, :DSENDECONSCOMAN, :DSCOMPLENDCOCOM,
    			:DSREFENDCONSCOM, :DSOBSCOMANDA, :IDORGCMDVENDA, :IDRETBALLOJA,
    			:NRPEDMOBILE, :NRLATPEDIDO, :NRLONGPEDIDO, :DSAREAATEND, 
    			:IDCOMANDAEXT, :NRCONTATOCON, :DSCONSUMIDOR, :IDIMPCUPEXP)";        
        
        if($order["order_type"] === 1){
            $IDORGCMDVENDA = "BAL_PED";
    	    $IDRETBALLOJA  = "C";
    	} else if($order["order_type"] === 2){
    	    $IDORGCMDVENDA = "TGO_PED";
    	    $IDRETBALLOJA  = "S";
    	} else if($order["order_type"] === 5) {
            $IDORGCMDVENDA = "DLV_PED";
    	    $IDRETBALLOJA  = "N";
    	} else {
    	    $IDORGCMDVENDA = "DLV_PED";
    	    $IDRETBALLOJA  = "N";
    	}

    	$consumerName = strtoupper($this->removeAcentos($order["customer"]["name"]));
    	
    	$params = [
    		":CDFILIAL"        => $generalParams["CDFILIAL"],
    		":NRVENDAREST"     => $order["NRVENDAREST"],
    		":NRCOMANDA"       => $order["NRCOMANDA"],
    		":NRCOMANDAEXT"    => $order["NRCOMANDAEXT"],
    		":CDLOJA"          => $generalParams["CDLOJA"],
    		":DSCOMANDA"       => "DLV_".$order["NRCOMANDA"],
    		":IDSTCOMANDA"     => $order["IDSTCOMANDA"],
    		":SGSEXOCON"       => "M",
    		":TXMOTIVCANCE"    => "",
    		":VRACRCOMANDA"    => $this->util->toFloat($deliveryFee),
    		":CDPAIS"          => "0055",
    		":SGESTADO"        => strtoupper($order["address"]["state"]), 
    		":CDMUNICIPIO"     => $consumer["CDMUNICIPIO"],
    		":NRCEPCONSCOMAND" => $order["address"]["zipCode"],
    		":CDBAIRRO"        => $consumer["CDBAIRRO"],
    		":DSBAIRRO"        => strtoupper($this->removeAcentos($order["address"]["neighborhood"])),
    		":NRENDECONSCOMAN" => (isset($order["address"]["number"]) && !empty($order["address"]["number"])) ? $order["address"]["number"] : "",
    		":DSENDECONSCOMAN" => substr($order["address"]["street"], 0, 44),
    		":DSCOMPLENDCOCOM" => $order["address"]["complement"],
    		":DSREFENDCONSCOM" => "",
    		":DSOBSCOMANDA"    => "",
    		":IDORGCMDVENDA"   => $IDORGCMDVENDA,
    		":IDRETBALLOJA"    => $IDRETBALLOJA,
    		":NRPEDMOBILE"     => "PED".$order["id"],
    		":NRLATPEDIDO"     => (isset($order["address"]["geoLat"]) && !empty($order["address"]["geoLat"])) ? $order["address"]["geoLat"] : "",
    		":NRLONGPEDIDO"    => (isset($order["address"]["geoLon"]) && !empty($order["address"]["geoLon"])) ? $order["address"]["geoLon"] : "",
    		":DSAREAATEND"     => $consumer["DSAREAATEND"],
    		":IDCOMANDAEXT"    => substr(($order["id"]), 0, 64),
    		":NRCONTATOCON"    => !empty($generalParams['NRCONTATOSAC']) ? $generalParams['NRCONTATOSAC'] : "",
    		":DSCONSUMIDOR"    => substr((isset($consumerName) ? $consumerName : ""), 0, 50),
    		":IDIMPCUPEXP"     => $order["IDIMPCUPEXP"]
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment ($order, $value, $generalParams, $consumer, $nsu)
    {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV, CDNSUTEFDLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV, :CDNSUTEFDLV)";

        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $order["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $order["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $consumer["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumer["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $order["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->util->toFloat($value),
	        ":NRCARTBANCODLV"  => null,
	        ":CDNSUTEFDLV"     => $nsu
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function isDuplicated ($generalParams, $order)
    {
        $sql = "
            SELECT C.NRPEDMOBILE

    		  FROM COMANDAVEN C

    		  JOIN VENDAREST V
    		    ON V.NRVENDAREST   = C.NRVENDAREST
    		   AND V.CDFILIAL      = C.CDFILIAL

    		 WHERE C.NRPEDMOBILE   = :NRPEDMOBILE
    		   AND V.CDFILIAL      = :CDFILIAL
    		   AND DTHRABERMESA    >= SYSDATE - 1
        ";
        
        $params = [
            ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRPEDMOBILE" => "PED".$order["id"]
        ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        return !empty($result);
    }
    
    /*
     * Função que busca dentro do json de TimeZones do projeto
     * o estado do cliente para buscar o Default Time Zone correto para o php
     * 
     */
    private function getCurrentTimezone ($state)
    {
        $timezones = array(
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        );
       
        return $timezones[$state];
    }
    
    public function validateOperator($generalParams)
    {
        $sql = "
            SELECT * FROM FILIOPER
             WHERE CDOPERADOR   = :CDOPERADOR
               AND CDFILIAL     = :CDFILIAL
        ";

        $params = [
            ":CDFILIAL"   => $generalParams["CDFILIAL"],
            ":CDOPERADOR" => $generalParams["CDOPERADOR"]
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function fetchIsCashierSinc($filial)
    {
        $sql = "
            SELECT  IDSINCCAIXADLV 
              FROM  CAIXA
             WHERE  CDFILIAL              = :CDFILIAL
               AND  UPPER(IDSINCCAIXADLV) = 'S'
        ";

        $params = [
            ":CDFILIAL" => $filial
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest, $orderId)
    {

        $taxSQL = "
            SELECT VRACRCOMANDA
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 2);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 2);
            $itemsSum = bcadd($itemsSum, $sub, 2);
        }

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $orderSum    = bcadd($itemsSum, $taxSum, 2);
        $paymentsSum = bcsub($entradasSum, $saidasSum, 2);

        if(bccomp($orderSum, $paymentsSum, 2) != 0) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu divergência de valores no pedido #{$orderId} da filial {$cdFilial}.\n");
            throw new Exception\ParamsException("
                Não foi possível processar o pedido. Existe uma divergencia entre os itens do pedido e o valor pago.<br>
                Valor dos itens: {$orderSum}.<br>
                Valor pago: {$paymentsSum}.");
        }
    }
    
    private function formatNumber($number)
    {
        return str_replace(",", ".", $number);
    }
    
    public function getOrderData($orderId, $cdFilial)
    {
        
        $sql = "
            SELECT CDFILIAL, NRVENDAREST, NRCOMANDA, IDSTCOMANDA, NRCOMANDAEXT
         
              FROM COMANDAVEN

             WHERE NRCOMANDAEXT = :NRCOMANDAEXT
               AND CDFILIAL     = :CDFILIAL
        ";
        
        $params = [
            ":CDFILIAL"      => $cdFilial,
            ":NRCOMANDAEXT"  => $orderId
            
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function checkIfIsPossibleToCancel($id)
    {
        $sql = "
        	SELECT C.CDFILIAL BRANCHID, C.NRCOMANDA ORDERNUMBER, C.NRVENDAREST SELLNUMBER
        	
        	  FROM COMANDAVEN C 

              JOIN VENDAREST VR
                ON VR.NRVENDAREST = C.NRVENDAREST
               AND VR.CDFILIAL    = C.CDFILIAL

         LEFT JOIN VENDA V
                ON C.CDFILIAL    = V.CDFILIAL
               AND C.NRVENDAREST = V.NRVENDAREST
               AND V.NRSEQVENDA IS NULL

             WHERE C.NRPEDMOBILE = :NRPEDMOBILE
               AND C.IDSTCOMANDA NOT IN ('4', 'X')
               AND C.IDORGCMDVENDA = 'DLV_IFO'
               AND VR.DTHRABERMESA >= SYSDATE - 1
          ORDER BY VR.DTHRABERMESA DESC
	    ";

        $params = [
            ':NRPEDMOBILE' => "PED".$id    
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function cancelOrder($orderData)
    {
        $sql = "
            UPDATE COMANDAVEN
               SET IDSTCOMANDA  = '4'
             
             WHERE CDFILIAL     = :CDFILIAL
               AND NRVENDAREST  = :NRVENDAREST
               AND NRCOMANDA    = :NRCOMANDA
               AND NRCOMANDAEXT = :NRCOMANDAEXT
               AND IDSTCOMANDA IN (1, 2, 3)
        ";
        
        $params = [
            ":CDFILIAL"     => $orderData['CDFILIAL'],
            ":NRVENDAREST"  => $orderData['NRVENDAREST'],
            ":NRCOMANDA"    => $orderData['NRCOMANDA'],
            ":NRCOMANDAEXT" => $orderData['NRCOMANDAEXT']
            
            
        ];
        
        return $this->connection->executeQuery($sql, $params);
    }
    
    private function removeAcentos($texto)
    
    {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
}
