<?php

namespace App\Service;

use App\Exception;

class ConsumerService {
    
    private $util;
    private $branchFactory;
    private $consumerFactory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\BranchFactory $branchFactory,
                                 \App\Factory\ConsumerFactory $consumerFactory) {

        $this->util            = $util;
        $this->branchFactory   = $branchFactory;
        $this->consumerFactory = $consumerFactory;
    }
    
    private function findConsumer($order, $standardClient, $generalParams)
    {
        $consumer = [];
        
        if (!empty($order["customer"]["email"])) {
            $consumer = $this->factory->getConsumerByEmail($order["customer"]["email"], $standardClient);
        }
        
        if (empty($consumer) && !empty($customer["telephone"])) {
            $phone = preg_replace("/\s+/", "", $customer["telephone"]);
            if (!empty($phone)) {
                $consumer = $this->factory->getConsumerByPhone($phone, $standardClient);
            }
        }
        
        if (empty($consumer)) {
            $consumer["customer"]["clientCode"] = $standardClient;
            if(empty($order["address"])) {
                throw new Exception\ValidationException("Não foi possível validar o endereço do pedido.");
            }
            
            $consumer = $this->registerConsumer($generalParams, $order["customer"], $order["address"]);
        }
        
        return $consumer;
    }
    
    public function registerConsumer ($generalParams, $customer, $address)
    {
        $stdClient = $this->branchFactory->getStandartClient($generalParams["CDFILIAL"]);
        $consumerT = $this->branchFactory->getStandartConsumerType($generalParams["CDFILIAL"]);
        $consumer  = "SDE".$this->util->newCode("CONSUMIDOR".$stdClient["CDCLIENTE"], 18);
        
        $consumerDetails["CDCONSUMIDOR"] = $consumer;
        $consumerDetails["CDMUNICIPIO"]  = $this->factory->getCityCode($address["state"], $address["city"]);
        
        if (empty($consumerDetails["CDMUNICIPIO"]))
            $consumerDetails["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
        
        $mergeCandidates = array($stdClient, $consumerT);
        
        foreach ($mergeCandidates as $candidate) 
            $consumerDetails = array_merge($consumerDetails, $candidate);
        
        $consumer = $this->factory->registerConsumer(array_merge($consumerDetails, $customer, $address));
        
        return $consumer;
    }
    
    public function processConsumer($generalParams, $order)
    {
        $consumer = $this->consumerFactory->getStandardConsumer($generalParams["CDFILIAL"]);

        $consumer["CDBAIRRO"] = $this->consumerFactory->getNeighborhoodCode($order["address"]);
        $consumer["DSBAIRRO"] = $order["address"]["neighborhood"];

        if (empty($consumer["CDMUNICIPIO"])) 
            $consumer["CDMUNICIPIO"] = $this->consumerFactory->getCityCode($order["address"]["state"], $order["address"]["city"]);

        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
            
        try {
            $consumer["DSAREAATEND"] = "";
            
            $areaAtend = $this->getDeliveryAreas($order["address"]["geoLat"], $order["address"]["geoLon"]);
            
            if((isset($areaAtend["INSIDE"])) && ($areaAtend["INSIDE"])) 
                $consumer["DSAREAATEND"] = $areaAtend["NAMEAREA"];

        } catch (\Exception $e) {
            $consumer["DSAREAATEND"] = "";
        } finally {
            return $consumer;
        } 
    }
    
    private function getDeliveryAreas($lat, $long)
    {
        
        $areas = $this->branchFactory->getDeliveryAreas();
        if(!$areas['error']) {
            $inside = [];

            $areas = $areas['areas'];
            foreach($areas as $cordinates) {
                $stringCoordinates   = $cordinates["COORDAREA"];
                $stringCoordinates   = str_replace("['","",$stringCoordinates);
                $stringCoordinates   = str_replace("']","",$stringCoordinates);
                $stringCoordinates   = str_replace("','",'$',$stringCoordinates);
                $polyCoordinates     = [];
                $explodedCoordinates = explode('$',$stringCoordinates);  
                
                foreach($explodedCoordinates as $latLongStr) {
                    $latLong = explode(',',$latLongStr);
                    array_push($polyCoordinates, $latLong);
                } 
                
                $inside = $this->inside($lat, $long, $polyCoordinates);
               
                if($inside) {
                    $inside = [
                        "INSIDE"    => $inside,
                        "NAMEAREA"  => $cordinates["DSAREAATEND"],
                    ];

                    return $inside;    
                }
            }

            return $inside;
        } else {
            throw new \Exception($areas["message"], 992); 
        }
    }
    
    private function inside($lat, $long, $vs)
    {
        $x = (float)$lat;
        $y = (float)$long;

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect) 
                $inside = !$inside;
        }
        
        return $inside;
    }
}
