<?php

namespace App\Service;

use App\Exception;

class ProductService {
    
    private $util;
    private $factory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $factory) {
        $this->util    = $util;
        $this->factory = $factory;
    }
    
    public function processProduct ($generalParams, $product, $order, &$nrSeqProdCom, &$ITCOMANDAVEN_SEQ) {

        // Valida se é Combo
        if (!empty($product["items"]) && count($product["items"]) > 0) {
            //Combo com código externo
            if (!empty($product["pos_reference"])) {
                $processedProduct = $this->processProductAsComboWithExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);
            } else { //Combo sem código externo
                $processedProduct = $this->processProductAsComboWithoutExtCode($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);    
            }
        } else {
            // Produto Simples
            $processedProduct = $this->processProductAsSimple($generalParams, $product, $order, $ITCOMANDAVEN_SEQ);
        }
        
        return $processedProduct;
    }
    
    private function processProductAsSimple ($generalParams, $product, $order, &$ITCOMANDAVEN_SEQ) {
        $simpleItens = [];

        if(!strpos($product["pos_reference"], "@")){
            throw new Exception\ParamsException("
                                O produto {$product['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
        }
        
        $exploded = explode("@", $product["pos_reference"]);
        
        if(count($exploded) != 2){
            throw new Exception\ParamsException("
                                O produto {$product['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
        }

        $product["pos_reference"] = $exploded[1];
        
        $product["quantity"] = (int) $product["quantity"];
        for ($i = 0 ; $i < $product["quantity"] ; $i++) {
            if (!isset($product["pos_reference"]) || empty($product["pos_reference"]))
                throw new Exception\ParamsException("
                    O produto {$product['pos_name']} não possui código externo e ele é obrigatório.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção junto a Pede Pronto.<br>
                    Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
            
            $productDetails = $this->factory->getProduct($product["pos_reference"]);
            
            if (empty($productDetails))
                throw new Exception\ParamsException("
                    Não existe um produto atrelado ao código externo informado. (COD 001)<br>
                    Produto: {$product['pos_name']}.<br>
                    Código externo: {$product['pos_reference']}.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção junto a Pede Pronto.<br>
                    Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
            
            $productDetails = array_merge($productDetails, $product);
            
            $productDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;
            
            $productDetails["TXPRODCOMVEN"] = "";
            
            $itemReturn = array(
                "CDFILIAL"       => $generalParams["CDFILIAL"],
                "NRVENDAREST"    => $order["NRVENDAREST"],
                "NRCOMANDA"      => $order["NRCOMANDA"],
                "NRPRODCOMVEN"   => $productDetails["NRPRODCOMVEN"],
                "CDPRODUTO"      => $productDetails["CDPRODUTO"],
                "NMPRODUTO"      => $productDetails["pos_name"],
                "QTPRODCOMVEN"   => 1,
                "VRPRECCOMVEN"   => $productDetails["unit_pos_price"],  
                "TXPRODCOMVEN"   => $productDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"   => isset($productDetails['VRDESCCOMVEN']) ? $productDetails['VRDESCCOMVEN'] : 0,
                "NRLUGARMESA"    => "01",
                "IDPRODIMPFIS"   => "N",
                "CDLOJA"         => $generalParams["CDLOJA"],
                "VRACRCOMVEN"    => 0,
                "NRSEQPRODCOM"   => null,
                "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN" => 0,
                "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO" => null,
                "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"    => $generalParams["CPF"],
                "DSOBSPEDDIGCMD" => null,
                "CDSENHAPED"     => $order["NRCOMANDAEXT"]
            );
        
            array_push($simpleItens, $itemReturn);
        }    
        return $simpleItens;

    }
    
    private function processProductAsComboWithExtCode ($generalParams, $combo, $order, &$ITCOMANDAVEN_SEQ, &$comboItens = array(), $CDPRODPROMO = null) {
        $comboItens = array();
        
       if(!strpos($combo["pos_reference"], "@")){
            throw new Exception\ParamsException("
                                O produto {$combo['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
        }

        $exploded = explode("@", $combo["pos_reference"]);
        
        if(count($exploded) != 2){
            throw new Exception\ParamsException("
                                O produto {$combo['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
        }

        $combo["pos_reference"] = $exploded[1];

        $combo["quantity"] = (int) $combo["quantity"];
        
        for ($i = 0 ; $i < $combo["quantity"] ; $i++) {
            if(!isset($combo["pos_reference"]) || empty($combo["pos_reference"]))
                throw new Exception\ParamsException("<p align='justify'>
                    O produto {$combo['pos_name']} foi cadastrado com preço e sem código externo. A obrigatoriedade do preço e do código externo dependem das regras abaixo:<br>
                    Caso o produto tenha preço, será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste produto, no formato 'xx.xx.xx.xx.xx'.<br>
                    Caso o produto seja um agrupador, ele não deve ter preço, nem código externo.<br>
                    O código externo (caso exista) deve ser igual ao cadastrado previamente no Odhen Retail.<br>
                    Para mais informações, entre em contato com a equipe de suporte da Teknisa.</p>");

            $comboDetails = $this->factory->getProduct($combo["pos_reference"]);

            if (empty($comboDetails))
                throw new Exception\ParamsException("
                    Não existe um produto atrelado ao código externo informado. (COD 002)<br>
                    Produto: {$combo['pos_name']}.<br>
                    Código externo: {$combo['pos_reference']}.<br>
                    Verifique no Odhen Retail o código externo deste produto e faça a correção no portal Pede Pronto.");

            $comboDetails = array_merge($comboDetails, $combo);
            
            $comboDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
            $ITCOMANDAVEN_SEQ++;
            
            $comboDetails["TXPRODCOMVEN"] = "";
            
            $itemReturn = array(
                "CDFILIAL"         => $generalParams["CDFILIAL"],
                "NRVENDAREST"      => $order["NRVENDAREST"],
                "NRCOMANDA"        => $order["NRCOMANDA"],
                "NRPRODCOMVEN"     => $comboDetails["NRPRODCOMVEN"],
                "CDPRODUTO"        => $comboDetails["CDPRODUTO"],
                "NMPRODUTO"        => $comboDetails["pos_name"],
                "QTPRODCOMVEN"     => 1,
                "VRPRECCOMVEN"     => $comboDetails["unit_pos_price"],  
                "TXPRODCOMVEN"     => $comboDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"     => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"     => isset($comboDetails['VRDESCCOMVEN']) ? $comboDetails['VRDESCCOMVEN'] : 0,
                "NRLUGARMESA"      => "01",
                "IDPRODIMPFIS"     => "N",
                "CDLOJA"           => $generalParams["CDLOJA"],
                "VRACRCOMVEN"      => 0,
                "NRSEQPRODCOM"     => $this->util->formatNrSeqProdCom($comboDetails["NRPRODCOMVEN"]),
                "NRSEQPRODCUP"     => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"   => 0,
                "CDCAIXACOLETOR"   => $generalParams["CDCAIXA"],
                //"CDPRODPROMOCAO"   => $CDPRODPROMO,
                "CDPRODPROMOCAO"   => null,
                "CDVENDEDOR"       => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"      => $generalParams["CPF"],
                "DSOBSPEDDIGCMD"   => null,
                "ITEM_OBSERVATION" => [],
                "CDSENHAPED"       => $order["NRCOMANDAEXT"]
            );

            $CDPRODPROMO = empty($CDPRODPROMO) ? $comboDetails["CDPRODUTO"] : $CDPRODPROMO;

            array_push($comboItens, $itemReturn);

            foreach ($combo["items"] as $comboItemAgrupador) {
                foreach ($comboItemAgrupador["items"] as $comboItem){
                    
                    $comboItem["quantity"] = (int) $comboItem["quantity"];
                    //========================== Observações Sem Preço ==========================

                    if(!empty($comboItem['pos_name']) && (empty($comboItem['unit_pos_price']) || $comboItem['unit_pos_price'] == "0.00")) {
                        // Observações tipo O e R
                        if(!isset($comboItem['pos_reference']) || empty($comboItem['pos_reference']))
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['pos_name']} foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'xx.xx.xx.xx.xx' ou no formato Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                O código externo deve ser igual ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
    

                        if(!strpos($comboItem["pos_reference"], "@")){
                            throw new Exception\ParamsException("
                                                O produto {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                        }

                        $explodedObs = explode("@", $comboItem["pos_reference"]);
                        
                        if(count($explodedObs) != 2){
                            throw new Exception\ParamsException("
                                                O produto {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                                O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                                Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                        }
                        
                        $observationCode = $explodedObs[1];

                        if(!strpos($observationCode, "|"))
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['pos_name']} foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$observationCode}.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
    
                        $validateObs = $this->factory->getObservation($observationCode);
                        
                        if($validateObs['IDCONTROLAOBS'] === "A"){
                            throw new Exception\ParamsException("
                                O subitem {$comboItem['pos_name']} foi cadastrado como um acréscimo utilizando o código no padrão XX|YY, mas está sem preço.<br>
                                Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código do item.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.<br>
                                O código externo e o valor do subitem (caso tenha) devem ser iguais ao cadastrado previamente no Retail.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
                        }

                        if(!count($validateObs))
                            throw new Exception\ParamsException("
                                Não existe uma observação atrelada ao código externo informado.<br>
                                Item do pedido: {$comboItem['pos_name']}.<br>
                                Código externo: {$comboItem['pos_reference']}.<br>
                                Verifique no Odhen Retail o código externo desta observação e faça a correção junto a Pede Pronto.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
                        
                        array_push($comboItens[0]['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],
                            'NRVENDAREST'     => $order["NRVENDAREST"],
                            'NRCOMANDA'       => $order["NRCOMANDA"],
                            'NRPRODCOMVEN'    => $comboDetails["NRPRODCOMVEN"],
                            'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],
                            'CDOCORR'         => $validateObs['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => $comboItem['quantity']
                        ]);
    
                        $this->processObservationWithoutPrice($comboDetails["NRPRODCOMVEN"], $validateObs['DSOCORR'], $comboItem['quantity'], $comboItens);
                        continue;                    
                    }
                    //======================= Fim de Observações Sem Preço ======================
                    
                    // QUANTIDADE DE ACRÉSCIMOS UM PARA CADA ACRÉSCIMO. SE TIVER 5 + BACON, SERÁ GRAVADO 1x BACON, 5 vezes
                    
                    if(!strpos($comboItem["pos_reference"], "@")){
                        throw new Exception\ParamsException("
                                            O subitem {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                            O subitem deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                            Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                    }
                    
                    $exploded = explode("@", $comboItem["pos_reference"]);

                    if(count($exploded) != 2){
                        throw new Exception\ParamsException("
                                            O subitem {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                                            O subitem deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                                            Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                    }
            
                    $comboItem["pos_reference"] = $exploded[1];
                    
                    if (strlen($comboItem["pos_reference"]) === 5 && substr($comboItem["pos_reference"], 2, 1) === '|' && $comboItem['unit_pos_price'] != "0.00"){
                        $itemDetails = $this->factory->getObservation($comboItem["pos_reference"]);
                    } else {
                        $itemDetails = $this->factory->getProduct($comboItem["pos_reference"]);
                    }

                    if (empty($itemDetails)){
                        if(strpos($comboItem['pos_reference'], "|")){
                            throw new Exception\ParamsException("
                                O subitem '{$comboItem['pos_name']}' possui preço ({$comboItem['unit_pos_price']}) e o código externo ({$comboItem['pos_reference']}) está no formato Código Grupo de Observação|Código de Observação (XX|YY).<br>
                                Caso o subitem seja uma observação ou remoção (por exemplo, 'Sem salada' ou 'Ao ponto'), você deve remover o preço deste subitem.<br>
                                Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), você deve corrigir o código externo deste subitem.<br>
                                O código externo e o valor do subitem devem ser iguais ao cadastrado previamente no Odhen Retail.<br>
                                Verifique no Retail o código externo e o valor desta observação e faça a correção junto a Pede Pronto.<br>
                                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
                        }
                        throw new Exception\ParamsException("
                            Não existe um produto atrelado ao código externo informado. (COD 003)<br>
                            Produto: {$comboItem['pos_name']}.<br>
                            Código externo: {$comboItem['pos_reference']}.<br>
                            Verifique no Retail o código externo e o valor desta observação e faça a correção junto a Pede Pronto.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
                    }
                        
                    $itemDetails = array_merge($itemDetails, $comboItem);
                    
                    for($j = 0; $j < $comboItem["quantity"]; $j++){
                        $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                        $ITCOMANDAVEN_SEQ++;
    
                        $itemDetails["TXPRODCOMVEN"] = (isset($itemDetails["observations"]) && !empty($itemDetails["observations"])) ? (" {$itemDetails['observations']}; ") : "";
                        
                        
                        $itemReturn = array(
                            "CDFILIAL"       => $generalParams["CDFILIAL"],
                            "NRVENDAREST"    => $order["NRVENDAREST"],
                            "NRCOMANDA"      => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1,
                            "VRPRECCOMVEN"   => $itemDetails["unit_pos_price"],  
                            "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                            "VRDESCCOMVEN"   => 0,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"    => 0,
                            "NRSEQPRODCOM"   => null,
                            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN" => 0,
                            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO" => $CDPRODPROMO,
                            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $generalParams["CPF"]),
                            "DSOBSPEDDIGCMD" => null,
                            "ITEM_OBSERVATION" => [],
                            "CDSENHAPED"     => $order["NRCOMANDAEXT"]
                        );
                        
                        if (strlen($comboItem["pos_reference"]) === 5 && substr($comboItem["pos_reference"], 2, 1) === '|' && $comboItem['unit_pos_price'] != "0.00"){

                            array_push($comboItens[0]['ITEM_OBSERVATION'], [
                                "CDFILIAL"        => $generalParams["CDFILIAL"],
                                "NRVENDAREST"     => $order["NRVENDAREST"],
                                "NRCOMANDA"       => $order["NRCOMANDA"],
                                "NRPRODCOMVEN"    => $itemDetails["NRPRODCOMVEN"],
                                "CDGRPOCOR"       => $itemDetails["CDGRPOCOR"],
                                "CDOCORR"         => $itemDetails["CDOCORR"],  
                                'NRPRODCOMVENOBS' => null, //$itemDetails["NRPRODCOMVEN"],  
                                'QTPRODCOMVENOBS' => $comboItem['quantity']
                            ]);
                        }
                        
                        array_push($comboItens, $itemReturn);
                    }
                }
            }
        }
        return $comboItens;
    }
    
    private function processProductAsComboWithoutExtCode ($generalParams, $combo, $order, &$ITCOMANDAVEN_SEQ, &$comboItens = array(), $CDPRODPROMO = null) {
        $comboItens = array();
        
        foreach ($combo["items"] as $comboItemAgrupador) {
            foreach ($comboItemAgrupador["items"] as $comboItem){
                
                if(!strpos($comboItem["pos_reference"], "@")){
                    throw new Exception\ParamsException("
                        O produto {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                        O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                        Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                }
                    
                $exploded = explode("@", $comboItem["pos_reference"]);
                
                if(count($exploded) != 2){
                    throw new Exception\ParamsException("
                        O produto {$comboItem['pos_name']} foi cadastrado fora do padrão aceito pela integração.<br>
                        O produto deve estar cadastrado no formato: nome_do_cliente@código_do_produto.<br>
                        Entre em contato com a equipe da Pede Pronto para fazer a correção no cadastro.");
                }
        
                $comboItem["pos_reference"] = $exploded[1];
                
                $comboItem["quantity"] = (int) $comboItem["quantity"];
                for ($i = 0 ; $i < $comboItem["quantity"]; $i++) {
                    
                    $itemDetails = $this->factory->getProduct($comboItem["pos_reference"]);
                    
                    if (empty($itemDetails))
                        throw new Exception\ParamsException("
                            Não existe um produto atrelado ao código externo informado. (COD 004)<br>
                            Produto: {$comboItem['pos_name']}.<br>
                            Código externo: {$comboItem['pos_reference']}.<br>
                            Verifique no Odhen Retail o código externo deste produto e faça a correção junto a Pede Pronto.<br>
                            Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Pede Pronto.");
                        
                    $itemDetails = array_merge($itemDetails, $comboItem);
                    
                    $itemDetails["NRPRODCOMVEN"] = str_pad((string) ($ITCOMANDAVEN_SEQ + 1), 6, '0', STR_PAD_LEFT);
                    $ITCOMANDAVEN_SEQ++;
                
                    $itemDetails["TXPRODCOMVEN"] = "";
                    
                    $itemReturn = array(
                    "CDFILIAL"       => $generalParams["CDFILIAL"],
                    "NRVENDAREST"    => $order["NRVENDAREST"],
                    "NRCOMANDA"      => $order["NRCOMANDA"],
                    "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                    "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                    "NMPRODUTO"      => $itemDetails["pos_name"],
                    "QTPRODCOMVEN"   => 1,
                    "VRPRECCOMVEN"   => $itemDetails["unit_pos_price"],  
                    "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                    "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                    "VRDESCCOMVEN"   => isset($itemDetails['VRDESCCOMVEN']) ? $itemDetails['VRDESCCOMVEN'] : 0,
                    "NRLUGARMESA"    => "01",
                    "IDPRODIMPFIS"   => "N",
                    "CDLOJA"         => $generalParams["CDLOJA"],
                    "VRACRCOMVEN"    => 0,
                    "NRSEQPRODCOM"   => null,
                    "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                    "VRPRECCLCOMVEN" => 0,
                    "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                    "CDPRODPROMOCAO" => $CDPRODPROMO,
                    "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                    "NRINSCRCONS"    => $generalParams["CPF"],
                    "DSOBSPEDDIGCMD" => null,
                    "CDSENHAPED"     => $order["NRCOMANDAEXT"]
                    );
                
                    $CDPRODPROMO = empty($CDPRODPROMO) ? $itemDetails["CDPRODUTO"] : $CDPRODPROMO;
                    array_push($comboItens, $itemReturn);
                }
            }
        }
        return $comboItens;
    }
    
    private function processObservationWithoutPrice($nrProdComVenKey, $observation, $quantity, &$products) {
        foreach($products as &$product) {
            if($nrProdComVenKey != $product['NRPRODCOMVEN']) {
                continue;
            }

            for($i = 0; $i < $quantity; $i++) {
                if(empty($product["TXPRODCOMVEN"])) {
                    $product["TXPRODCOMVEN"] = " $observation; ";
                } else {
                    $product["TXPRODCOMVEN"] .= "$observation; ";
                }
            }            
        }
        
        if(isset($product)) {
            unset($product);
        }
    }
    
    public function insertProduct ($product) {
        $this->factory->insertProduct($product);
    }

    public function shareDiscount($initialDiscount, &$products) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if((bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1) && empty($product["reedem_id"])) { 
                $productsTotalValue = bcadd($productsTotalValue, $product["VRPRECCOMVEN"], 2);
                array_push($idxToApplyDiscount, $i);
            }
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]["VRPRECCOMVEN"], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;
            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }
    
        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }
        
        $leftDiscount = "0.00";
        foreach($products as &$product) {
            if( (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 1) || (bccomp($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2) == 0) )  {
                $leftDiscount = bcadd($leftDiscount, bcsub($product['VRDESCCOMVEN'], $product['VRPRECCOMVEN'], 2), 2);
                $leftDiscount = bcadd($leftDiscount, "0.01", 2);
                
                $product['VRDESCCOMVEN'] = bcsub($product['VRDESCCOMVEN'], $leftDiscount, 2);
            }
        }
        
        if(isset($product))
            unset($product);

        foreach($products as &$product){
            $maxAddDisc = (bcsub(bcsub($product['VRPRECCOMVEN'], "0.01", 2), $product['VRDESCCOMVEN'], 2));
            if($maxAddDisc >= $leftDiscount) {
                $addDisc = $leftDiscount;
            } else{
                $addDisc = $maxAddDisc;
            }
            
            $product['VRDESCCOMVEN'] = bcadd($product['VRDESCCOMVEN'], $addDisc, 2);
            $leftDiscount = bcsub($leftDiscount, $addDisc, 2);
        }

        if(isset($product))
            unset($product);
            
        return $products;
    }
    
    public function insertObservation($observation) {
        $this->factory->insertObservation($observation);
    }

}
